$(document).ready(function(){
    movement.initiateMovement();
    connectToWebsocket();
});

/**
 * Une classe pour controller le mouvement de SVG
 */
var movement = {
    carRotation: 0,         // La dégrée de rotation actuelle
    rotationValue: 1,       // Le changement de rotation lors de la réccption d'une commande
    rotationValueInPlay: 2,       // Le changement de rotation lors de la réccption d'une commande mais dans le mode Play
    initiateMovement: function(){
        $('#front-arrow').hide();
        $('#back-arrow').hide();
        $('#left-front-arrow').hide();
        $('#left-back-arrow').hide();
        $('#right-front-arrow').hide();
        $('#right-back-arrow').hide();
    },
    setRotation: function(degree){
        movement.carRotation = degree;
        $('#svg').css('transform', `rotate(${movement.carRotation}deg)`)
    },
    // Faire tourner la voiture par une dégrée
    rotateCar: function(degree){
        movement.carRotation += degree;
        $('#svg').css('transform', `rotate(${movement.carRotation}deg)`)
    },
    // Les deux roues roulent tout droit
    rotateBothFront: function(){
        movement.initiateMovement();
        $('#front-arrow').show();
        $('#left-front-arrow').show();
        $('#right-front-arrow').show();
    },
    // Les deux roues roulent en arrière
    rotateBothBack: function(){
        movement.initiateMovement();
        $('#back-arrow').show();
        $('#left-back-arrow').show();
        $('#right-back-arrow').show();
    },
    // La voiture tourne à droite (seulement la roue de gauche roule tout droit)
    rotateLeftFront: function(){
        movement.initiateMovement();
        $('#front-arrow').show();
        $('#left-front-arrow').show();
        if(controlPanel.isPlaying){
            movement.rotateCar(movement.rotationValueInPlay);
        }else{
            movement.rotateCar(movement.rotationValue);
        }
    },
    // La voiture tourne à gauche (seulement la roue de droite roule tout droit)
    rotateRightFront: function(){
        movement.initiateMovement();
        $('#front-arrow').show();
        $('#right-front-arrow').show();
        if(controlPanel.isPlaying){
            movement.rotateCar(-movement.rotationValueInPlay);
        }else{
            movement.rotateCar(-movement.rotationValue);
        }
    },
    // La voiture tourne à droite mais en arrière (seulement la roue de gauche roule en arrière)
    rotateLeftBack: function(){
        movement.initiateMovement();
        $('#back-arrow').show();
        $('#left-back-arrow').show();
        if(controlPanel.isPlaying){
            movement.rotateCar(-movement.rotationValueInPlay);
        }else{
            movement.rotateCar(-movement.rotationValue);
        }
    },
    // La voiture tourne à gauche mais en arrière (seulement la roue de droite roule en arrière)
    rotateRightBack: function(){
        movement.initiateMovement();
        $('#back-arrow').show();
        $('#right-back-arrow').show();
        if(controlPanel.isPlaying){
            movement.rotateCar(movement.rotationValueInPlay);
        }else{
            movement.rotateCar(movement.rotationValue);
        }
    },
    // Les deux roues sont arrêtées
    stopRotation: function(){
        movement.initiateMovement();
    }
};

/**
 * Classe pour l'affichage de vitesse
 */
var screen = {
    // Mettre à jour l'écrat à partir d'une distance donnée
    updateDisplay: function(distance){
        let color = "green";
        if(distance < 31){
            color = "red";
        }
        $('#distance')
            .css('background', color)
            .text(distance + "cm");
    }
};

function connectToWebsocket () {
    // if user is running mozilla then use it's built-in WebSocket
    window.WebSocket = window.WebSocket || window.MozWebSocket;
  
    var connection = new WebSocket('ws://127.0.0.1:3001');
  
    connection.onopen = function () {
      // connection is opened and ready to use
    };
  
    connection.onerror = function (error) {
      // an error occurred when sending/receiving data
    };
  
    connection.onmessage = function (message) {
      // try to decode json (I assume that each message
      // from server is json)
      try {
        var json = JSON.parse(message.data);
        if(json.type == "distance" && (controlPanel.isCarMoving || controlPanel.isPlaying)){
            screen.updateDisplay(json.value);
        }else if(json.type == "wheels" && (controlPanel.isCarMoving || controlPanel.isPlaying)){
            if(json.value == 1){
                movement.rotateBothFront();
            }else if(json.value == 2){
                movement.rotateBothBack();
            }else if(json.value == 3){
                movement.stopRotation();
            }else if(json.value == 4){
                movement.rotateLeftFront();
            }else if(json.value == 5){
                movement.rotateLeftBack();
            }else if(json.value == 6){
                movement.rotateRightFront();
            }else{
                movement.rotateRightBack();
            }
        }else if(json.type == "connection"){
            if(json.value == true){
                controlPanel.updateArduinoStatus(true);
            }else{
                controlPanel.updateArduinoStatus(false);
            }
        }else if(json.type == "websocket" && controlPanel.isPlaying){
            if(json.value == 1){
                controlPanel.updatePlayStatus(false);
                movement.setRotation(0);
                movement.stopRotation();
                screen.updateDisplay(0);
            }
        }
        console.log(json);
      } catch (e) {
        console.log('This doesn\'t look like a valid JSON: ',
            message.data);
        return;
      }
      // handle incoming message
    };
};

var controlPanel = {
    isArduinoConnected: false,  // Une variable si l'Arduino est connecté
    isCarMoving: false,         // Une variable si la voiture roule
    isRecording: false,         // Une variable si on enregistre
    isPlaying: false,           // Une variable si on joue un enregistrement
    // Une fonction pour modifier la variable précédante ainsi que le style du bouton
    updateArduinoStatus: function(status){
        controlPanel.isArduinoConnected = status;
        if(status){
            $('.arduino-connected').removeClass('red').addClass('green');
        }else{
            $('.arduino-connected').addClass('red').removeClass('green');
        }
    },
    // Une fonction pour faire rouler/arreter la voiture (la variable precedante)
    startStartArduinoButtonClicked: function(){
        if(controlPanel.isArduinoConnected){
            if(controlPanel.isCarMoving){
                // arreter la voiture
                controlPanel.isCarMoving = false;
                $.get('http://localhost:3000/commands/stop-start-car/1', function(){
                    movement.stopRotation();
                    screen.updateDisplay(0);
                    $('.arduino-start-stop').removeClass('red').addClass('green').text('Rouler');
                });
            }else{
                if(controlPanel.isPlaying){
                    controlPanel.updatePlayStatus(false);
                    movement.setRotation(0);
                    movement.stopRotation();
                    screen.updateDisplay(0);
                    $.get('http://localhost:3000/commands/change-mode/1', function(){
                        // faire rouler la voiture
                        controlPanel.isCarMoving = true;
                        $.get('http://localhost:3000/commands/stop-start-car/2', function(){
                            $('.arduino-start-stop').removeClass('green').addClass('red').text('S\'arreter');
                        });
                    });
                }else{
                    // faire rouler la voiture
                    controlPanel.isCarMoving = true;
                    $.get('http://localhost:3000/commands/stop-start-car/2', function(){
                        $('.arduino-start-stop').removeClass('green').addClass('red').text('S\'arreter');
                    });
                }
            }
        }else{
            alert("L'Arduino n'est pas connecté!")
        }
    },
    updateRecordStatus: function(status){
        controlPanel.isRecording = status;
        if(!status){
            $('.arduino-record').removeClass('red').addClass('green').text('Enregistrer');
        }else{
            $('.arduino-record').addClass('red').removeClass('green').text('Arreter l\'enregistrement');
        }
    },
    recordButtonClicked: function(){
        if(controlPanel.isArduinoConnected){
            let wasPlaying = controlPanel.isPlaying;
            controlPanel.updatePlayStatus(false);
            if(controlPanel.isRecording){
                controlPanel.updateRecordStatus(!controlPanel.isRecording);
                $.get('http://localhost:3000/commands/change-mode/1', function(){
                });
            }else{
                controlPanel.updateRecordStatus(!controlPanel.isRecording);
                $.get('http://localhost:3000/commands/change-mode/2', function(){
                    if(wasPlaying){
                        movement.setRotation(0);
                        movement.stopRotation();
                        screen.updateDisplay(0);
                    }
                });
            }
        }else{
            alert("L'Arduino n'est pas connecté!")
        }
    },
    updatePlayStatus: function(status){
        controlPanel.isPlaying = status;
        if(!status){
            $('.arduino-play-record').removeClass('red').addClass('green').text('Jouer dernier enregistrement');
        }else{
            $('.arduino-play-record').addClass('red').removeClass('green').text('Arreter la lecteur');
        }
    },
    playLastRecordButtonClicked: function(){
        if(controlPanel.isArduinoConnected){
            if(controlPanel.isPlaying){
                controlPanel.updatePlayStatus(!controlPanel.isPlaying);
                movement.setRotation(0);
                movement.stopRotation();
                screen.updateDisplay(0)
            }else{
                controlPanel.updatePlayStatus(!controlPanel.isPlaying);
                movement.setRotation(0);
                controlPanel.isCarMoving = false;
                movement.stopRotation();
                screen.updateDisplay(0)
                controlPanel.updateRecordStatus(false);
                $('.arduino-start-stop').removeClass('red').addClass('green').text('Rouler');
                $.get('http://localhost:3000/commands/change-mode/3', function(){
                });
            }
        }else{
            alert("L'Arduino n'est pas connecté!")
        }
    }
};